/* 
   cadaver, command-line DAV client
   Copyright (C) 1999, Joe Orton <joe@orton.demon.co.uk>
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
   $Id: cadaver.c,v 1.4 1999/09/29 23:26:13 joe Exp $
*/

#include <config.h>

#include <sys/types.h>

#include <stdio.h>

#ifdef HAVE_READLINE_H
#include <readline.h>
#elif HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif 

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <ctype.h>
#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

#include <signal.h>

#include "getopt.h"

#include "frontend.h"
#include "protocol.h"
#include "httpdav.h"
#include "string_utils.h"
#include "basename.h"
#include "dirname.h"
#include "common.h"
#include "dates.h"

static void usage(void);
static void parse_args( int argc, char *argv[],
		       struct proto_host *server, char *path );
static char * read_command(void);

static struct proto_host server;
static char path[BUFSIZ] = {0};

static char *progname;

static void usage() {
    printf( "Usage: %s hostname[:port] path\n", progname );
}

static void set_abspath( const char *newpath ) {
    strcpy( path, newpath );
    if( path[strlen(path)-1] != '/' )
	strcat( path, "/" );
}

static void set_relpath( char *rel ) {
    strcat( path, rel );
    if( path[strlen(path)-1] != '/' )
	strcat( path, "/" );
}
	
static void version( void ) {
    printf( PACKAGE " " VERSION "\n" );
}
	    
static void parse_args( int argc, char *argv[], 
		       struct proto_host *server, char *path ) {
    static const struct option opts[] = {
	{ "version", no_argument, NULL, 'V' },
	{ "help", no_argument, NULL, 'h' },
	{ 0, 0, 0, 0 }
    };
    int optc;
    char *pnt;
    while( (optc = getopt_long( argc, argv, "hV", opts, NULL )) != -1 ) {
	switch( optc ) {
	case 'h': usage( ); exit( -1 );
	case 'V': version( ); exit( -1 );
	case '?': 
	default:
	    printf( "Try `%s --help' for more information.\n", progname );
	    exit(-1);
	}
    }
    if( argc < 3 ) { 
	usage( );
	exit( -1 );
    }
    pnt = strchr( argv[1], ':' );
    if( pnt != NULL && (strlen(pnt+1)>0) ) {
	*pnt = '\0';
	server->port = atoi( pnt+1 );
    } else {
	server->port = 80;
    }
    set_abspath( argv[2] );
    server->hostname = argv[1];
    server->username = NULL;
    server->password = NULL;
}

static char * read_command(void) {
    static char prompt[BUFSIZ];
    snprintf( prompt, BUFSIZ, "dav:%s> ", path );
    return readline( prompt ); 
}

static void execute_ls( const char *path ) {
    int ret;
    struct proto_file *files, *current;
    ret = dav_fetch( path, 0, &files );
    if( ret == PROTO_OK ) {
	for( current = files; current!=NULL; current = current->next ) {
	    char *date = rfc1123_date( current->modtime ), *restype;
	    switch( current->type ) {
	    case proto_file: restype = "Resource"; break;
	    case proto_link: restype = "Reference"; break;
	    case proto_dir: restype = "Collection"; break;
	    default: restype = "Unknown"; break;
	    }		    
	    printf( "%10s: %-20s %10d  %s\n", restype, current->filename, 
		    current->size, date );
	    free( date );
	}
    } else {
	printf( "Could not ls.\n" );
    }
}

static int execute_command( const char *cmd ) {
    char **tokens;
    static int lemon_counter = 0;
    tokens = split_string( cmd, ' ', NULL, NULL );
    if( tokens[0] == NULL ) return 0;
    if( strcasecmp( *tokens, "cd" ) == 0 ) {
	if( tokens[1] == NULL ) return -1;
	if( *tokens[1] == '/' ) { 
	    printf( "abscd to %s\n", tokens[1] );
	    set_abspath( tokens[1] );
	} else {
	    printf( "relcd to %s\n", tokens[1] );
	    set_relpath( tokens[1] );
	}
    } else if( strcasecmp( tokens[0], "ls" ) == 0 ) {
	execute_ls( path );
    } else if( strcasecmp( tokens[0], "debug" ) == 0 ) {
	if( tokens[1] != NULL )
	    debug_mask = atoi( tokens[1] );
	printf( "Debugging level now: %d\n", debug_mask );
    } else if( strcasecmp( tokens[0], "quit" ) == 0 ) {
	return -1;
    } else if( strcasecmp( tokens[0], "help" ) == 0 ) {
	printf( 
"Commands: \n"
"  ls              List current directory contents.\n"
"  cd /path/       Change to specified ABSOLUTE path.\n"
"  debug [x]       Set (or just display) debugging level.\n"
"  quit            Exit program.\n" );
    } else if( strcmp( tokens[0], "" ) == 0 ) {
	if( (++lemon_counter % 15) == 0 )
	    printf( "Type something, I'm getting bored of this!\n" );
    } else {
	printf( "You what? Try `help'.\n" );
    }
    return 0;
}

int main( int argc, char *argv[] ) {
    int ret = 0;
    progname = argv[0];
    debug_mask = 0;
    signal( SIGPIPE, SIG_IGN );
    
    parse_args( argc, argv, &server, path );

    if( http_init( path, &server, NULL ) != PROTO_OK ) {
	return -1;
    }
    while(ret == 0) {
	char *cmd;
	cmd = read_command();
	ret = execute_command(cmd);
	free( cmd );
    } 
    http_finish( );
    return 0;
}

void fe_connection( fe_conn_status status ) {
    switch( status ) {
    case fe_namelookup:
	printf( "Looking up hostname...\n" );
	break;
    case fe_connecting:
	printf( "Connecting to server...\n" );
	break;
    case fe_connected:
	printf( "Connected.\n" );
	break;
    }	    
}

void fe_transfer_progress( size_t progress, size_t total ) {
}

void fe_login( const char **username, const char **password ) {
    *username = readline( "Username: " );
    *password = getpass( "Password: " );
}
